@@ -24,10 +24,14 @@ import os
 import signal
 import stat
 import tempfile
+import uuid
+import errno
 from urllib.parse import urlparse
 
 import grpc
 
+from .. import _yaml
+
 from .._protos.google.bytestream import bytestream_pb2, bytestream_pb2_grpc
 from .._protos.build.bazel.remote.execution.v2 import remote_execution_pb2, remote_execution_pb2_grpc
 from .._protos.buildstream.v2 import buildstream_pb2, buildstream_pb2_grpc
@@ -59,6 +63,8 @@ class CASCache(ArtifactCache):
         os.makedirs(os.path.join(self.casdir, 'refs', 'heads'), exist_ok=True)
         os.makedirs(os.path.join(self.casdir, 'objects'), exist_ok=True)
 
+        self._calculate_cache_quota()
+
         self._enable_push = enable_push
 
         # Per-project list of _CASRemote instances.
@@ -81,7 +87,8 @@ class CASCache(ArtifactCache):
 
         tree = self.resolve_ref(ref, update_mtime=True)
 
-        dest = os.path.join(self.extractdir, element._get_project().name, element.normal_name, tree.hash)
+        dest = os.path.join(self.extractdir, element._get_project().name,
+                            element.normal_name, tree.hash)
         if os.path.isdir(dest):
             # artifact has already been extracted
             return dest
@@ -99,7 +106,7 @@ class CASCache(ArtifactCache):
                 #
                 # If rename fails with these errors, another process beat
                 # us to it so just ignore.
-                if e.errno not in [os.errno.ENOTEMPTY, os.errno.EEXIST]:
+                if e.errno not in [errno.ENOTEMPTY, errno.EEXIST]:
                     raise ArtifactError("Failed to extract artifact for ref '{}': {}"
                                         .format(ref, e)) from e
 
@@ -309,8 +316,11 @@ class CASCache(ArtifactCache):
                     # Upload any blobs missing on the server
                     skipped_remote = False
                     for digest in missing_blobs.values():
-                        def request_stream():
-                            resource_name = os.path.join(digest.hash, str(digest.size_bytes))
+                        uuid_ = uuid.uuid4()
+                        resource_name = '/'.join(['uploads', str(uuid_), 'blobs',
+                                                  digest.hash, str(digest.size_bytes)])
+
+                        def request_stream(resname):
                             with open(self.objpath(digest), 'rb') as f:
                                 assert os.fstat(f.fileno()).st_size == digest.size_bytes
                                 offset = 0
@@ -324,12 +334,12 @@ class CASCache(ArtifactCache):
                                     request.write_offset = offset
                                     # max. 64 kB chunks
                                     request.data = f.read(chunk_size)
-                                    request.resource_name = resource_name
+                                    request.resource_name = resname
                                     request.finish_write = remaining <= 0
                                     yield request
                                     offset += chunk_size
                                     finished = request.finish_write
-                        response = remote.bytestream.Write(request_stream())
+                        response = remote.bytestream.Write(request_stream(resource_name))
 
                     request = buildstream_pb2.UpdateReferenceRequest()
                     request.keys.append(ref)
@@ -518,6 +528,25 @@ class CASCache(ArtifactCache):
     #
     def remove(self, ref, *, defer_prune=False):
 
+        # Remove extract if not used by other ref
+        tree = self.resolve_ref(ref)
+        ref_name, ref_hash = os.path.split(ref)
+        extract = os.path.join(self.extractdir, ref_name, tree.hash)
+        keys_file = os.path.join(extract, 'meta', 'keys.yaml')
+        if os.path.exists(keys_file):
+            keys_meta = _yaml.load(keys_file)
+            keys = [keys_meta['strong'], keys_meta['weak']]
+            remove_extract = True
+            for other_hash in keys:
+                if other_hash == ref_hash:
+                    continue
+                remove_extract = False
+                break
+
+            if remove_extract:
+                utils._force_rmtree(extract)
+
+        # Remove cache ref
         refpath = self._refpath(ref)
         if not os.path.exists(refpath):
             raise ArtifactError("Could not find artifact for ref '{}'".format(ref))
@@ -747,7 +776,7 @@ class CASCache(ArtifactCache):
             yield from self._required_blobs(dirnode.digest)
 
     def _fetch_blob(self, remote, digest, out):
-        resource_name = os.path.join(digest.hash, str(digest.size_bytes))
+        resource_name = '/'.join(['blobs', digest.hash, str(digest.size_bytes)])
         request = bytestream_pb2.ReadRequest()
         request.resource_name = resource_name
         request.read_offset = 0
