@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 #
 #  Copyright (C) 2018 Codethink Limited
 #
@@ -22,6 +21,7 @@ from blessings import Terminal
 
 # Import a widget internal for formatting time codes
 from .widget import TimeCode
+from .._scheduler import ElementJob
 
 
 # Status()
@@ -33,34 +33,36 @@ from .widget import TimeCode
 # to a terminal, or if the terminal does not support ANSI escape codes.
 #
 # Args:
+#    context (Context): The Context
 #    content_profile (Profile): Formatting profile for content text
 #    format_profile (Profile): Formatting profile for formatting text
 #    success_profile (Profile): Formatting profile for success text
 #    error_profile (Profile): Formatting profile for error text
-#    pipeline (Pipeline): The Pipeline
-#    scheduler (Scheduler): The Scheduler
+#    stream (Stream): The Stream
 #    colors (bool): Whether to print the ANSI color codes in the output
 #
 class Status():
 
-    def __init__(self, content_profile, format_profile,
+    def __init__(self, context,
+                 content_profile, format_profile,
                  success_profile, error_profile,
-                 pipeline, scheduler, colors=False):
+                 stream, colors=False):
 
+        self._context = context
         self._content_profile = content_profile
         self._format_profile = format_profile
         self._success_profile = success_profile
         self._error_profile = error_profile
-        self._pipeline = pipeline
-        self._scheduler = scheduler
+        self._stream = stream
         self._jobs = []
         self._last_lines = 0  # Number of status lines we last printed to console
         self._term = Terminal()
         self._spacing = 1
         self._colors = colors
-        self._header = _StatusHeader(content_profile, format_profile,
+        self._header = _StatusHeader(context,
+                                     content_profile, format_profile,
                                      success_profile, error_profile,
-                                     pipeline, scheduler)
+                                     stream)
 
         self._term_width, _ = click.get_terminal_size()
         self._alloc_lines = 0
@@ -76,9 +78,9 @@ class Status():
     #    element (Element): The element of the job to track
     #    action_name (str): The action name for this job
     #
-    def add_job(self, element, action_name):
-        elapsed = self._scheduler.elapsed_time()
-        job = _StatusJob(element, action_name, self._content_profile, self._format_profile, elapsed)
+    def add_job(self, job):
+        elapsed = self._stream.elapsed_time
+        job = _StatusJob(self._context, job, self._content_profile, self._format_profile, elapsed)
         self._jobs.append(job)
         self._need_alloc = True
 
@@ -90,7 +92,13 @@ class Status():
     #    element (Element): The element of the job to track
     #    action_name (str): The action name for this job
     #
-    def remove_job(self, element, action_name):
+    def remove_job(self, job):
+        action_name = job.action_name
+        if not isinstance(job, ElementJob):
+            element = None
+        else:
+            element = job.element
+
         self._jobs = [
             job for job in self._jobs
             if not (job.element is element and
@@ -133,7 +141,7 @@ class Status():
         if not self._term.does_styling:
             return
 
-        elapsed = self._scheduler.elapsed_time()
+        elapsed = self._stream.elapsed_time
 
         self.clear()
         self._check_term_width()
@@ -242,18 +250,19 @@ class Status():
 # A delegate object for rendering the header part of the Status() widget
 #
 # Args:
+#    context (Context): The Context
 #    content_profile (Profile): Formatting profile for content text
 #    format_profile (Profile): Formatting profile for formatting text
 #    success_profile (Profile): Formatting profile for success text
 #    error_profile (Profile): Formatting profile for error text
-#    pipeline (Pipeline): The Pipeline
-#    scheduler (Scheduler): The Scheduler
+#    stream (Stream): The Stream
 #
 class _StatusHeader():
 
-    def __init__(self, content_profile, format_profile,
+    def __init__(self, context,
+                 content_profile, format_profile,
                  success_profile, error_profile,
-                 pipeline, scheduler):
+                 stream):
 
         #
         # Public members
@@ -267,24 +276,25 @@ class _StatusHeader():
         self._format_profile = format_profile
         self._success_profile = success_profile
         self._error_profile = error_profile
-        self._pipeline = pipeline
-        self._scheduler = scheduler
-        self._time_code = TimeCode(content_profile, format_profile)
+        self._stream = stream
+        self._time_code = TimeCode(context, content_profile, format_profile)
+        self._context = context
 
     def render(self, line_length, elapsed):
+        project = self._context.get_toplevel_project()
         line_length = max(line_length, 80)
         size = 0
         text = ''
 
-        session = str(self._pipeline.session_elements)
-        total = str(self._pipeline.total_elements)
+        session = str(len(self._stream.session_elements))
+        total = str(len(self._stream.total_elements))
 
-        # Format and calculate size for pipeline target and overall time code
+        # Format and calculate size for target and overall time code
         size += len(total) + len(session) + 4  # Size for (N/N) with a leading space
         size += 8  # Size of time code
-        size += len(self._pipeline.project.name) + 1
+        size += len(project.name) + 1
         text += self._time_code.render_time(elapsed)
-        text += ' ' + self._content_profile.fmt(self._pipeline.project.name)
+        text += ' ' + self._content_profile.fmt(project.name)
         text += ' ' + self._format_profile.fmt('(') + \
                 self._content_profile.fmt(session) + \
                 self._format_profile.fmt('/') + \
@@ -296,10 +306,10 @@ class _StatusHeader():
         text = ''
 
         # Format and calculate size for each queue progress
-        for queue in self._scheduler.queues:
+        for queue in self._stream.queues:
 
             # Add spacing
-            if self._scheduler.queues.index(queue) > 0:
+            if self._stream.queues.index(queue) > 0:
                 size += 2
                 text += self._format_profile.fmt('→ ')
 
@@ -354,15 +364,20 @@ class _StatusHeader():
 # A delegate object for rendering a job in the status area
 #
 # Args:
-#    element (Element): The element being processed
-#    action_name (str): The name of the action
+#    context (Context): The Context
+#    job (Job): The job being processed
 #    content_profile (Profile): Formatting profile for content text
 #    format_profile (Profile): Formatting profile for formatting text
-#    elapsed (datetime): The offset of the scheduler when this job is created
+#    elapsed (datetime): The offset into the session when this job is created
 #
 class _StatusJob():
 
-    def __init__(self, element, action_name, content_profile, format_profile, elapsed):
+    def __init__(self, context, job, content_profile, format_profile, elapsed):
+        action_name = job.action_name
+        if not isinstance(job, ElementJob):
+            element = None
+        else:
+            element = job.element
 
         #
         # Public members
@@ -370,6 +385,7 @@ class _StatusJob():
         self.element = element            # The Element
         self.action_name = action_name    # The action name
         self.size = None                  # The number of characters required to render
+        self.full_name = element._get_full_name() if element else action_name
 
         #
         # Private members
@@ -377,12 +393,12 @@ class _StatusJob():
         self._offset = elapsed
         self._content_profile = content_profile
         self._format_profile = format_profile
-        self._time_code = TimeCode(content_profile, format_profile)
+        self._time_code = TimeCode(context, content_profile, format_profile)
 
         # Calculate the size needed to display
         self.size = 10  # Size of time code with brackets
         self.size += len(action_name)
-        self.size += len(element._get_full_name())
+        self.size += len(self.full_name)
         self.size += 3  # '[' + ':' + ']'
 
     # render()
@@ -399,7 +415,7 @@ class _StatusJob():
             self._format_profile.fmt(']')
 
         # Add padding after the display name, before terminating ']'
-        name = self.element._get_full_name() + (' ' * padding)
+        name = self.full_name + (' ' * padding)
         text += self._format_profile.fmt('[') + \
             self._content_profile.fmt(self.action_name) + \
             self._format_profile.fmt(':') + \
