@@ -1,5 +1,5 @@
 #
-#  Copyright (C) 2017 Codethink Limited
+#  Copyright (C) 2018 Codethink Limited
 #
 #  This program is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU Lesser General Public
@@ -335,6 +335,51 @@ class Plugin():
         """
         return _yaml.node_get(node, expected_type, member_name, default_value=default)
 
+    def node_get_project_path(self, node, key, *,
+                              check_is_file=False, check_is_dir=False):
+        """Fetches a project path from a dictionary node and validates it
+
+        Paths are asserted to never lead to a directory outside of the
+        project directory. In addition, paths can not point to symbolic
+        links, fifos, sockets and block/character devices.
+
+        The `check_is_file` and `check_is_dir` parameters can be used to
+        perform additional validations on the path. Note that an
+        exception will always be raised if both parameters are set to
+        ``True``.
+
+        Args:
+           node (dict): A dictionary loaded from YAML
+           key (str): The key whose value contains a path to validate
+           check_is_file (bool): If ``True`` an error will also be raised
+                                 if path does not point to a regular file.
+                                 Defaults to ``False``
+           check_is_dir (bool): If ``True`` an error will also be raised
+                                if path does not point to a directory.
+                                Defaults to ``False``
+
+        Returns:
+           (str): The project path
+
+        Raises:
+           :class:`.LoadError`: In the case that the project path is not
+                                valid or does not exist
+
+        *Since: 1.2*
+
+        **Example:**
+
+        .. code:: python
+
+          path = self.node_get_project_path(node, 'path')
+
+        """
+
+        return _yaml.node_get_project_path(node, key,
+                                           self.__project.directory,
+                                           check_is_file=check_is_file,
+                                           check_is_dir=check_is_dir)
+
     def node_validate(self, node, valid_keys):
         """This should be used in :func:`~buildstream.plugin.Plugin.configure`
         implementations to assert that users have only entered
@@ -478,13 +523,15 @@ class Plugin():
                                            silent_nested=silent_nested):
             yield
 
-    def call(self, *popenargs, fail=None, **kwargs):
+    def call(self, *popenargs, fail=None, fail_temporarily=False, **kwargs):
         """A wrapper for subprocess.call()
 
         Args:
            popenargs (list): Popen() arguments
            fail (str): A message to display if the process returns
                        a non zero exit code
+           fail_temporarily (bool): Whether any exceptions should
+                                    be raised as temporary. (*Since: 1.2*)
            rest_of_args (kwargs): Remaining arguments to subprocess.call()
 
         Returns:
@@ -507,16 +554,18 @@ class Plugin():
               "Failed to download ponies from {}".format(
                   self.mirror_directory))
         """
-        exit_code, _ = self.__call(*popenargs, fail=fail, **kwargs)
+        exit_code, _ = self.__call(*popenargs, fail=fail, fail_temporarily=fail_temporarily, **kwargs)
         return exit_code
 
-    def check_output(self, *popenargs, fail=None, **kwargs):
+    def check_output(self, *popenargs, fail=None, fail_temporarily=False, **kwargs):
         """A wrapper for subprocess.check_output()
 
         Args:
            popenargs (list): Popen() arguments
            fail (str): A message to display if the process returns
                        a non zero exit code
+           fail_temporarily (bool): Whether any exceptions should
+                                    be raised as temporary. (*Since: 1.2*)
            rest_of_args (kwargs): Remaining arguments to subprocess.call()
 
         Returns:
@@ -555,7 +604,7 @@ class Plugin():
               raise SourceError(
                   fmt.format(plugin=self, track=tracking)) from e
         """
-        return self.__call(*popenargs, collect_stdout=True, fail=fail, **kwargs)
+        return self.__call(*popenargs, collect_stdout=True, fail=fail, fail_temporarily=fail_temporarily, **kwargs)
 
     #############################################################
     #            Private Methods used in BuildStream            #
@@ -619,7 +668,7 @@ class Plugin():
 
     # Internal subprocess implementation for the call() and check_output() APIs
     #
-    def __call(self, *popenargs, collect_stdout=False, fail=None, **kwargs):
+    def __call(self, *popenargs, collect_stdout=False, fail=None, fail_temporarily=False, **kwargs):
 
         with self._output_file() as output_file:
             if 'stdout' not in kwargs:
@@ -634,7 +683,8 @@ class Plugin():
             exit_code, output = utils._call(*popenargs, **kwargs)
 
             if fail and exit_code:
-                raise PluginError("{plugin}: {message}".format(plugin=self, message=fail))
+                raise PluginError("{plugin}: {message}".format(plugin=self, message=fail),
+                                  temporary=fail_temporarily)
 
         return (exit_code, output)
 
