@@ -19,8 +19,6 @@ import pytest
 #
 from _pytest.capture import MultiCapture, FDCapture
 
-from tests.testutils.site import IS_LINUX
-
 # Import the main cli entrypoint
 from buildstream._frontend import cli as bst_cli
 from buildstream import _yaml
@@ -159,14 +157,14 @@ class Result():
         return list(tracked)
 
     def get_pushed_elements(self):
-        pushed = re.findall(r'\[\s*push:(\S+)\s*\]\s*START\s*Pushing artifact', self.stderr)
+        pushed = re.findall(r'\[\s*push:(\S+)\s*\]\s*INFO\s*Pushed artifact', self.stderr)
         if pushed is None:
             return []
 
         return list(pushed)
 
     def get_pulled_elements(self):
-        pulled = re.findall(r'\[\s*pull:(\S+)\s*\]\s*START', self.stderr)
+        pulled = re.findall(r'\[\s*pull:(\S+)\s*\]\s*INFO\s*Downloaded artifact', self.stderr)
         if pulled is None:
             return []
 
@@ -203,10 +201,7 @@ class Cli():
     def remove_artifact_from_cache(self, project, element_name):
         cache_dir = os.path.join(project, 'cache', 'artifacts')
 
-        if IS_LINUX:
-            cache_dir = os.path.join(cache_dir, 'ostree', 'refs', 'heads')
-        else:
-            cache_dir = os.path.join(cache_dir, 'tar')
+        cache_dir = os.path.join(cache_dir, 'cas', 'refs', 'heads')
 
         cache_dir = os.path.splitext(os.path.join(cache_dir, 'test', element_name))[0]
         shutil.rmtree(cache_dir)
@@ -337,7 +332,6 @@ class Cli():
             'show',
             '--deps', 'none',
             '--format', '%{state}',
-            '--downloadable',
             element_name
         ])
         result.assert_success()
